package owl;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import edu.stanford.smi.protege.model.Frame;
import edu.stanford.smi.protege.model.KnowledgeBase;
import edu.stanford.smi.protege.model.Project;
import edu.stanford.smi.protege.model.Slot;
import edu.stanford.smi.protege.model.WidgetDescriptor;
import edu.stanford.smi.protege.query.api.QueryApi;
import edu.stanford.smi.protege.query.api.QueryConfiguration;
import edu.stanford.smi.protege.query.indexer.IndexMechanism;
import edu.stanford.smi.protege.query.querytypes.impl.PhoneticQuery;
import edu.stanford.smi.protegex.owl.model.OWLModel;

public class TestLuceneQuery {
    public final static String PROJECT_FILE="/Users/tredmond/Shared/ontologies/simple/pizza.pprj";
    
    
    @SuppressWarnings("unchecked")
    public static void displayErrors(List errors) {
        if (!errors.isEmpty()) {
            for (Object error : errors) {
                if (error instanceof Throwable) {
                    ((Throwable) error).printStackTrace();
                }
                else {
                    System.out.println("Error " + error);
                }
            }
        }
    }
    
    @SuppressWarnings("unchecked")
    private static void addTab(Project project) {
        List errors = new ArrayList();
        
        WidgetDescriptor tabWidgetDescriptor = project.getTabWidgetDescriptor("edu.stanford.smi.protege.query.LuceneQueryPlugin");
        tabWidgetDescriptor.setVisible(true);

        Collection widgets = project.getTabWidgetDescriptors();
        project.setTabWidgetDescriptorOrder(widgets);
        
        project.save(errors);
        displayErrors(errors);
    }
    
    private static void testQuery(KnowledgeBase kb) {
        Slot slot = kb.getSystemFrames().getNameSlot();
        Collection<Frame> results = null;
        results = kb.executeQuery(new PhoneticQuery(slot, "sheazeypeasah"));
        if (results == null || results.size() != 1) {
            System.out.println("test of search failed");
        }
        else {
            System.out.println("Search succeeded (" + results.iterator().next().getBrowserText() + ")");
        }
    }

    private static void alterConfiguration(KnowledgeBase kb, QueryConfiguration config) {
        config.setBaseIndexPath("/tmp/lucene");
        Set<IndexMechanism> indexers = new HashSet<IndexMechanism>();
        indexers.add(IndexMechanism.PHONETIX_INDICIES);
        config.setIndexers(indexers);
        Set<Slot> slots = new HashSet<Slot>();
        slots.add(kb.getSystemFrames().getNameSlot());
    }
    
    private static void indexOntology(KnowledgeBase kb) {
        QueryApi api = new QueryApi(kb);
        QueryConfiguration config = new QueryConfiguration(kb);
        
        alterConfiguration(kb, config);
        
        api.install();
        api.index(config);
    }
    
    
    @SuppressWarnings("unchecked")
    public static void main(String args[]) {
        List errors = new ArrayList();
        Project project = new Project(PROJECT_FILE, errors);
        displayErrors(errors);
        OWLModel om = (OWLModel) project.getKnowledgeBase();

        indexOntology(om);
        testQuery(om);
        addTab(project);
    }

}
